/*
    FreeRTOS V7.4.2 - Copyright (C) 2013 Real Time Engineers Ltd.

    FEATURES AND PORTS ARE ADDED TO FREERTOS ALL THE TIME.  PLEASE VISIT
    http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS tutorial books are available in pdf and paperback.        *
     *    Complete, revised, and edited pdf reference manuals are also       *
     *    available.                                                         *
     *                                                                       *
     *    Purchasing FreeRTOS documentation will not only help you, by       *
     *    ensuring you get running as quickly as possible and with an        *
     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
     *    the FreeRTOS project to continue with its mission of providing     *
     *    professional grade, cross platform, de facto standard solutions    *
     *    for microcontrollers - completely free of charge!                  *
     *                                                                       *
     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
     *                                                                       *
     *    Thank you for using FreeRTOS, and thank you for your support!      *
     *                                                                       *
    ***************************************************************************


    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.

    >>>>>>NOTE<<<<<< The modification to the GPL is included to allow you to
    distribute a combined work that includes FreeRTOS without being obliged to
    provide the source code for proprietary components outside of the FreeRTOS
    kernel.

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
    details. You should have received a copy of the GNU General Public License
    and the FreeRTOS license exception along with FreeRTOS; if not it can be
    viewed here: http://www.freertos.org/a00114.html and also obtained by
    writing to Real Time Engineers Ltd., contact details for whom are available
    on the FreeRTOS WEB site.

    1 tab == 4 spaces!

    ***************************************************************************
     *                                                                       *
     *    Having a problem?  Start by reading the FAQ "My application does   *
     *    not run, what could be wrong?"                                     *
     *                                                                       *
     *    http://www.FreeRTOS.org/FAQHelp.html                               *
     *                                                                       *
    ***************************************************************************


    http://www.FreeRTOS.org - Documentation, books, training, latest versions,
    license and Real Time Engineers Ltd. contact details.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, and our new
    fully thread aware and reentrant UDP/IP stack.

    http://www.OpenRTOS.com - Real Time Engineers ltd license FreeRTOS to High
    Integrity Systems, who sell the code with commercial support,
    indemnification and middleware, under the OpenRTOS brand.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.
*/


/*-----------------------------------------------------------
 * Implementation of functions defined in portable.h for the ARM7 port.
 *
 * Components that can be compiled to either ARM or THUMB mode are
 * contained in this file.  The ISR routines, which can only be compiled
 * to ARM mode are contained in portISR.c.
 *----------------------------------------------------------*/

/*
 * Modified by A.Heinze and M.Bhme/HAMEG Instruments GmbH
 * - Atmel SAMA5D31 registers
 * - per task reent structs
 * - stack checksum
 */

/* Standard includes. */
#include <stdlib.h>
#include <string.h>
#include <reent.h>


/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"


#include <assert.h>
#include <debug.h>

/* Processor constants. */

/* Constants required to setup the task context. */
#define portINITIAL_SPSR				( ( portSTACK_TYPE ) 0x1f ) /* System mode, ARM mode, interrupts enabled. */
#define portTHUMB_MODE_BIT				( ( portSTACK_TYPE ) 0x20 )
#define portINSTRUCTION_SIZE			( ( portSTACK_TYPE ) 4 )
#define portNO_CRITICAL_SECTION_NESTING	( ( portSTACK_TYPE ) 0 )


/*-----------------------------------------------------------*/

#if defined FREE_RTOS_REENT_SIZE && DEBUG_EDITOR == DEBUG_EDITOR_MBC
// damit man im Trace32 die Aufzhlung anschauen kann
const enum contextStruct		portContextStructList = REG_CRITNEST_VAR;
// wieviele Register braucht man fr das reent
const unsigned int				portReentSize = FREE_RTOS_REENT_SIZE;
#endif

/* Setup the timer to generate the tick interrupts. */
//static void prvSetupTimerInterrupt( void );

/*
 * The scheduler can only be started from ARM mode, so
 * vPortISRStartFirstSTask() is defined in portISR.c.
 */
extern void vPortISRStartFirstTask( void );


#if defined DEBUG_RTOS_STACK_CHECK_CRC
// simple Prfsumme zur Erkennung von offensichtlichen fremden nderungen am Stack eines Tasks
// FIXME: ordentliche CRC32 nutzen
portStack_TYPE xPortCalcStackCrc (const portStack_TYPE *pxTopOfStack)
{
	const unsigned char	*pData = (void *)pxTopOfStack;
	portStack_TYPE		result = 0;
	int					count = (REG_SYS_NO - REG_STACK_CRC - 1) * sizeof(portSTACK_TYPE);

	// Summe ber alle Bytes des Stacks
	while (count > 0)
	{
		result += *pData;
		pData++;
		count--;
	}
	return result;
}

#endif

/*-----------------------------------------------------------*/

void pxPortDeleteCurrentTask (void)
{
	vTaskDelete (NULL);

	// hier darf man einfach nicht hinkommen
	assert (0);
	while (1);
}
/*-----------------------------------------------------------*/

/*
 * Initialise the stack of a task to look exactly as if a call to
 * portSAVE_CONTEXT had been called.
 *
 * See header file for description.
 */
portSTACK_TYPE *pxPortInitialiseStack( portSTACK_TYPE *pxTopOfStack, pdTASK_CODE pxCode, void *pvParameters )
{
portSTACK_TYPE *pxOriginalTOS;

#if FREERTOS_REENT_CHANGE_POINTER
	//wir wollen immer nur den Zeiger auf die reent-Struktur umschalten
	//wir legen die reent-Struktur also an das Ende des Stacks, merken uns den Zeiger auf diese
	//und bei einem Taskwechsel wird dieser Zeiger
	// bei SAVE_CONTEXT von portCurrentTaskReentPtr auf den Stack an [REG_REENT_POINTER] kopiert
	// bei RESTORE_CONTEXT von [REG_REENT_POINTER] an portCurrentTaskReentPtr kopiert, so dass sysCall_getreent(CPU_IRQ_PRIOR_SYS) diesen liefert

	portSTACK_TYPE *reent_ptr;

	// Zeiger auf den Anfang setzen
	pxTopOfStack -= FREE_RTOS_REENT_SIZE;
	// alles lschen
	memset(pxTopOfStack, 0, FREE_RTOS_REENT_SIZE * sizeof(portSTACK_TYPE));
	// reent-Struktur fr den Task initialisieren
	_REENT_INIT_PTR((struct _reent *)pxTopOfStack);

	//den Zeiger auf das reent Struct merken
	reent_ptr = pxTopOfStack;

	//auf 8 Byte Alignment achten
	if ((portSTACK_TYPE)pxTopOfStack & 0x7)
		pxTopOfStack--;

	//jetz kanns normal weitergehen
#endif

	pxOriginalTOS = pxTopOfStack;

	// !!! Anzahl und Inhalt der Stackvariablen siehe enum contextStruct !!!

	/* To ensure asserts in tasks.c don't fail, although in this case the assert
	is not really required. */
	pxTopOfStack--;

	/* Setup the initial stack of the task.  The stack is set exactly as
	expected by the portRESTORE_CONTEXT() macro. */

	/* First on the stack is the return address - which in this case is the
	start of the task.  The offset is added to make the return address appear
	as it would within an IRQ ISR. */
	*pxTopOfStack = ( portSTACK_TYPE ) pxCode; // + portINSTRUCTION_SIZE;	/* IRQ_LR */

	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) pxPortDeleteCurrentTask;	/* R14 = SYS_LR */ // -> wenn der Task ans Ende kommt, wird er automatisch gelscht
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) pxOriginalTOS; /* Stack used when task starts goes in R13 = SYS_SP */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x12121212;	/* R12 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x11111111;	/* R11 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x10101010;	/* R10 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x09090909;	/* R9 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x08080808;	/* R8 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x07070707;	/* R7 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x06060606;	/* R6 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x05050505;	/* R5 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x04040404;	/* R4 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x03030303;	/* R3 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x02020202;	/* R2 */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x01010101;	/* R1 */

	/* When the task starts is will expect to find the function parameter in R0. */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) pvParameters; /* R0 */

	/* The next thing onto the stack is the status register, which is set for system mode, with interrupts enabled. */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) portINITIAL_SPSR;
#ifdef THUMB_INTERWORK
	/* We want the task to start in thumb mode. */
	*pxTopOfStack |= portTHUMB_MODE_BIT;
#endif

	// die VPU-Regs
	for (unsigned int i = 0; i < (REG_SYS_S31 - REG_SYS_S00 + 1); i++)
	{
		pxTopOfStack--;
		*pxTopOfStack = ( portSTACK_TYPE ) i;	/* S31 - S0*/
	}
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x00000000;		/* FPSCR */
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) 0x40000000;		/* FPEXC */

	/* Some optimisation levels use the stack differently to others.  This
	means the interrupt flags cannot always be stored on the stack and will
	instead be stored in a variable, which is then saved as part of the
	tasks context. */
	pxTopOfStack--;
	*pxTopOfStack = portNO_CRITICAL_SECTION_NESTING;

#if FREERTOS_REENT_COPY_STRUCT
	// Zeiger auf den Anfang setzen
	pxTopOfStack -= (REG_REENT_END - REG_REENT_START + 1);
	// alles lschen
	memset(pxTopOfStack, 0, (REG_REENT_END - REG_REENT_START + 1) * sizeof(portSTACK_TYPE));
	// reent-Struktur fr den Task initialisieren
	_REENT_INIT_PTR((struct _reent *)pxTopOfStack);
#elif FREERTOS_REENT_CHANGE_POINTER
	//den Pointer auf das reent-Struct des Tasks eintragen
	pxTopOfStack--;
	*pxTopOfStack = ( portSTACK_TYPE ) reent_ptr;
#endif

#if FREE_RTOS_REG_STACK_ALIGNMENT
	// Ausrichtung auf 8 Byte erforderlich
	pxTopOfStack--;
	*pxTopOfStack = 0;	/* STACK_ALIGNMENT */
#endif
	// CRC ausrechnen und eintragen
	pxTopOfStack--;
#if defined DEBUG_RTOS_STACK_CHECK_CRC
	*pxTopOfStack = xPortCalcStackCrc (&pxTopOfStack[1]);	/* CRC */
#else
	*pxTopOfStack = 0;	/* CRC */
#endif

	return pxTopOfStack;
}
/*-----------------------------------------------------------*/

portBASE_TYPE xPortStartScheduler( void )
{
	/* Start the first task. */
	vPortISRStartFirstTask();

	/* Should not NEVER(!!!) get here! */
	assert (0);
	while(1);
	return 0;
}
/*-----------------------------------------------------------*/

void vPortEndScheduler( void )
{
	/* It is unlikely that the ARM port will require this function as there
	is nothing to return to.  */
}
/*-----------------------------------------------------------*/
